import os, errno

def ensure_dir_exists(path):
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else: raise

def get_backupfile(cfg, key):
    fname = os.path.basename(get_rcfile(cfg, key))
    backupfile = os.path.join(cfg.get('dirs', 'backup-dir'), \
        '%s-%s.tgz' % (cfg.args['PACKAGE'].replace('/', '_'), fname))
    return backupfile

def get_rcfile(cfg, key):
    rcfile = cfg.get('rcfiles', key)
    if rcfile and rcfile[-1] != '/':
        return os.path.abspath(rcfile)
    if rcfile and rcfile[-1] == '/':
        return os.path.join(os.path.abspath(rcfile), '.%s' % key)
    return os.path.join(cfg.get('dirs', 'target'), '.%s' % key)

def add_checksum(filename): pass

def verify_checksum(filename): pass



import os
import tarfile
from contextlib import closing
from . import helpers

def gzip_then_store(filename, archivename):
    with closing(tarfile.open(archivename, 'w:gz')) as tar:
        tar.add(filename)
    helpers.add_checksum(archivename)
    return archivename

def Backup(cfg, rcfiles):
    backupfiles = []
    backupdir = cfg.get('dirs', 'backup-dir')
    helpers.ensure_dir_exists(backupdir)
    for rc in rcfiles:
        orifile = helpers.get_rcfile(cfg, rc)
        if os.path.lexists(orifile) and not os.path.islink(orifile):
            fname = os.path.basename(orifile)
            archivename = helpers.get_backupfile(cfg, rc)
            backupfiles.append(gzip_then_store(orifile, archivename))
    return backupfiles



import os, shutil
from archie import helpers, backup

def create_symlink(cfg, pkg, rcfiles):
    linkfile = []
    for rc in rcfiles:
        ourfile = os.path.abspath(os.path.join(pkg, rc))
        orifile = helpers.get_rcfile(cfg, rc)
        if os.path.lexists(orifile):
            try:
                os.unlink(orifile)
            except OSError:
                if os.path.isdir(orifile):
                    shutil.rmtree(orifile)
        else:
            helpers.ensure_dir_exists(os.path.dirname(orifile))
        os.symlink(ourfile, orifile)
        linkfile.append((orifile, ourfile))
    return linkfile

def Install(cfg):
    pkg = cfg.get('args', 'PACKAGE')
    rcfiles = cfg.options('rcfiles')
    helpers.ensure_dir_exists(cfg.get('dirs', 'target'))
    backup.Backup(cfg, rcfiles)
    return create_symlink(cfg, pkg, rcfiles)



try:
    from ConfigParser import SafeConfigParser
except ImportError:
    from configparser import ConfigParser as SafeConfigParser

class Config:
    def __init__(self, configfile, args):
        self.args = args
        try:
            self.conf = SafeConfigParser(allow_no_value = True)
        except TypeError:
            self.conf = SafeConfigParser()
        self.conf.readfp(open(configfile))

    def get(self, section, key):
        if section == 'args':
            return self.args[key]
        if section == 'dirs':
            return self.__defaults_(key)
        return self.conf.get(section, key, raw = False, vars = {
            'target':       self.__defaults_('target'),
            'backup-dir':   self.__defaults_('backup-dir') })

    def options(self, section):
        return self.conf.options(section)

    def __defaults_(self, key):
        return self.args['--' + key] or self.conf.get('dirs', key, raw = False)



import os
import tarfile
from contextlib import closing
from archie import helpers

def find_backup(cfg):
    files = []
    rcfiles = cfg.options('rcfiles')
    for rc in rcfiles:
        backup = helpers.get_backupfile(cfg, rc)
        rcfile = helpers.get_rcfile(cfg, rc)
        if os.path.lexists(backup) and tarfile.is_tarfile(backup):
            files.append((backup, rcfile))
    return files

def gunzip_and_restore(cfg, backupfiles):
    for backup, rc in backupfiles:
        if os.path.islink(rc):
            os.unlink(rc)
        with closing(tarfile.open(backup, 'r:gz')) as tar:
            tar.extractall('/')
    return backupfiles

def Restore(cfg):
    backupfiles = find_backup(cfg)
    return gunzip_and_restore(cfg, backupfiles)



import os
from .configuration import Config
from . import handlers as cmd


VERSION = '0.0.3'
APPNAME = 'archie'


def main(args):
    configfile = args['--config'] or os.path.join(args['PACKAGE'], 'a.rc')
    cfg = Config(configfile, args)

    if args['install']:
        result = cmd.Install(cfg)
    elif args['restore']:
        result = cmd.Restore(cfg)

    for item in result:
        print(item)
